<?php

namespace app\action;

use Exception;
use libs\db\Db;
use libs\listener\Event;

class SetTrayLogin
{
    public function execute($post)
    {
        // 验证数据
        $param = check_valid($post['action'], [
            ['tray', 'string', '托盘号'],
            ['lot', 'string', '流水号'],
            ['batch', 'string', '批次号'],
            ['device_code', 'string', '设备号'],
            ['workingsubclass', 'string', '工序单元'],
            ['battery_ids', 'array', '电池条码数组']
        ], $post['param']);

        // 提交的电池结果数据
        $date = date('Y-m-d H:i:s');
        $battery_ids = $param['battery_ids'];
        $param['CHANNEL'] = [];
        $param['DEVICE_CODE'] = [];
        $param['START_TIME'] = [];
        $param['FINISH_TIME'] = [];

        try {
            // 判断是否存在不是字符串'0'的情况，约束托盘空位值只能传字符串'0'而不是其他值
            $filteredArray = array_filter($battery_ids, function ($value) {
                return $value !== '0' && empty($value);
            }, ARRAY_FILTER_USE_BOTH);
            if (count($filteredArray) > 0) {
                $keys = implode(',', array_keys($filteredArray));
                throw new Exception("电池条码数组[battery_ids]以下位置[{$keys}]中存在不合法的值，如[0, '', null, NAN]等，电池条码值只能是[字符串'0'或者真实条码]");
            }

            // 判断电池是否存在重复值
            $filteredArray = array_filter($battery_ids, function ($value) {
                return !empty($value);
            });
            $repeat_batterys = array_keys(array_count_values($filteredArray), 2);
            if (count($repeat_batterys) > 0) {
                $repeat_keys = implode(',', array_keys($repeat_batterys));
                throw new Exception("电池条码数组[battery_ids]存在重复的电池条码[{$repeat_keys}]，请检查！");
            }

            list($tray, $lot, $batch, $workingsubclass, $device_code) = [
                $param['tray'],
                $param['lot'],
                $param['batch'],
                $param['workingsubclass'],
                $param['device_code']
            ];

            // 1、根据电池数组赋予通道数据
            $param['CHANNEL'] = array_map(function ($value) {
                return $value + 1;
            }, array_keys($battery_ids));
            // 赋予设备结果参数数组
            $param['DEVICE_CODE'] = array_pad($param['DEVICE_CODE'], count($battery_ids), $device_code);
            // 赋予开始时间结果参数数组
            $param['START_TIME'] = array_pad($param['START_TIME'], count($battery_ids), $date);
            // 赋予完成时间结果参数数组
            $param['FINISH_TIME'] = array_pad($param['FINISH_TIME'], count($battery_ids), $date);

            // 判断流水号是否是合法的时间戳
            $date = date('Y-m-d H:i:s', intval($lot));
            if ($date === false || strtotime($date) === false) {
                throw new Exception("该流水号[{$lot}]不是一个有效的时间戳");
            }

            // 批次数据是否存在
            $sql = sprintf(
                "SELECT a.flow_id, b.process
                    FROM hf_mes_production_planning_management_batch AS a
                    INNER JOIN hf_mes_technology_flow AS b ON b.id = a.flow_id
                    WHERE a.batch='%s' LIMIT 1;",
                $batch
            );
            $ret = Db::fetch($sql);
            if (empty($ret)) {
                throw new Exception("该批次[{$batch}]不存在");
            }
            list($flow_id, $flow_process) = [$ret['flow_id'], json_decode($ret['process'], true)];

            // 拼接当前的process_code
            $current_process_code = "{$flow_id}_{$workingsubclass}";
            // 判断第一工序是否与当前工序一致
            if (empty($flow_process)) {
                throw new Exception("工序数据不存在");
            }
            if ($current_process_code != $flow_process[0][2]) {
                throw new Exception("当前工序[{$current_process_code}]与系统当前工序[{$flow_process[0][2]}]不一致");
            }
            $current_process_idx = $flow_process[0][0];
            // 下一工序编码
            $next_process_code = -1;
            if (count($flow_process) > 1) {
                $next_process_code = $flow_process[1][2];
            }

            // 获取工序结果参数
            $sql = sprintf(
                "SELECT id
                    FROM hf_mes_technology_process
                    WHERE code='%s' LIMIT 1;",
                $current_process_code
            );
            $ret = Db::fetch($sql);
            $process_id = $ret['id'];

            $sql = sprintf(
                "SELECT code,field_type,is_upload
                    FROM hf_mes_technology_process_result_param
                    WHERE process_id='%s';",
                $process_id
            );
            $result_param_need = Db::query($sql);

            // 刷选出需要设备上传的结果参数
            $result_param_need_params = array_column($result_param_need, 'code');
            $result_param_need_column = array_column($result_param_need, NULL, 'code');

            // 验证具体的结果数据有没有上传
            $param_keys = array_keys($param);
            // 判断包含在$result_param_need_params数组中存在而在$param_keys数组中不存在，以此来判断结果参数是否全部上传了
            $diff_param_keys = array_diff($result_param_need_params, $param_keys);
            if (count($diff_param_keys) > 0) {
                $diff_param_keys_str = implode(',', $diff_param_keys);
                throw new Exception("结果参数[{$diff_param_keys_str}]没有上传，请检查！");
            }
            // 验证结果参数的类型是否正确
            $insert_bkv_keys_change = [];
            foreach ($result_param_need as $item) {
                if (!in_array($item['code'], $param_keys)) {
                    throw new Exception("结果参数[{$item['code']}]没有上传，请检查！");
                }
                // 验证结果参数是否是数组上传
                if (!is_array($param[$item['code']])) {
                    throw new Exception("结果参数[{$item['code']}]需要以数组的形式上传，例如：['1', '2']");
                }
                // 结果参数数组
                $p_vals = $param[$item['code']];
                // 结果参数数据类型
                $p_class = $result_param_need_column[$item['code']]['field_type'];
                foreach ($p_vals as $i => $p_val) {
                    if ($p_class == 'TIMESTAMP') {
                        if ($p_val != "" && date('Y-m-d H:i:s', strtotime($p_val)) != $p_val) {
                            throw new Exception(sprintf(
                                "结果数据[%s]第[%s]位置的数值[%s]的数据类型与系统[%s]类型要求不相符，请检查！【要求数据类型格式比如为：'2022-11-11 09:00:00'】",
                                $item['param'],
                                $i + 1,
                                $p_val,
                                $p_class
                            ));
                        }
                    } elseif ($p_class == 'FLOAT') {
                        if ($p_val != "" && !is_numeric($p_val)) {
                            throw new Exception(sprintf(
                                "结果数据[%s]第[%s]位置的数值[%s]的数据类型与系统[%s]类型要求不相符，请检查！【要求数据类型格式比如为： 12.8】",
                                $item['param'],
                                $i + 1,
                                $p_val,
                                $p_class
                            ));
                        }
                    } elseif ($p_class == 'INT') {
                        if ($p_val != "" && !is_int($p_val)) {
                            throw new Exception(sprintf(
                                "结果数据[%s]第[%s]位置的数值[%s]的数据类型与系统[%s]类型要求不相符，请检查！【要求数据类型格式比如为： 12】",
                                $item['param'],
                                $i + 1,
                                $p_val,
                                $p_class
                            ));
                        }
                    }
                }

                // BKV动态结果参数
                array_push($insert_bkv_keys_change, "{$current_process_code}.{$item['code']}");
            }


            // 获取子批次
            $sql = sprintf(
                "SELECT a.subbatch,a.batch_id
                    FROM hf_mes_production_planning_management_subbatch AS a
                    INNER JOIN hf_mes_production_planning_management_batch b ON b.id = a.batch_id
                    WHERE b.batch='%s' ORDER BY a.id DESC LIMIT 1;",
                $batch
            );
            $ret = Db::fetch($sql);
            if (empty($ret)) {
                throw new Exception("不存在子批次数据");
            }
            $batch_id = $ret['batch_id'];
            $subbatch = $ret['subbatch'];

            // 2、判断电池在原来托盘是否还处于激活和同一批次，不允许存在相同电池
            // 3、组装BKV的新增数据
            // 4、组装battery_map的新增数据
            // 5、组装process_log的新增数据
            $insert_bkv_keys = 'battery_id,tray,lot,active,process_code,process_idx,next_process_code,pid,uuid';
            if (!empty($insert_bkv_keys_change)) {
                $insert_bkv_keys = $insert_bkv_keys . ',"' . implode('","', $insert_bkv_keys_change) . '"';
            }
            $insert_bkv_vals = [];

            $insert_battery_map_keys = 'battery_unique_value,batch,subbatch,tray,lot,battery_id,pid,uuid';
            $insert_battery_map_vals = [];

            $insert_process_log_data = [];

            $actual_input_battery_count = 0;

            foreach ($battery_ids as $battery_index => $battery_id) {
                // 判断如果查询不到battery_map的数据，pid赋予-2，uuid赋予空数组
                $pid = '-2';
                $uuid = json_encode([generateUniqueId()]);

                // 2、判断电池在原来托盘是否还处于激活和同一批次，不允许存在相同电池
                if ($battery_id !== '0') {
                    // 过滤掉0的情况
                    $sql = sprintf(
                        "SELECT batch,tray,lot,subbatch,pid,uuid
                    FROM hf_mes_production_battery_map
                    WHERE battery_id='%s' ORDER BY id DESC LIMIT 1;",
                        $battery_id
                    );
                    $ret = Db::fetch($sql);
                    if (!empty($ret)) {
                        // 原来托盘号
                        $original_tray = $ret['tray'];

                        $pid = !empty($ret['pid']) ? $ret['pid'] : $pid;
                        $uuid = !empty($ret['uuid']) ? $ret['uuid'] : $uuid;

                        $sql = sprintf(
                            "SELECT id
                    FROM \"%s\"
                    WHERE battery_id='%s' AND tray='%s' AND lot='%s' AND active=1 LIMIT 1;",
                            config('app.bkv_prefix') . $ret['subbatch'],
                            $battery_id,
                            $original_tray,
                            $ret['lot']
                        );
                        $ret = Db::fetch($sql);
                        if (!empty($ret) && $original_tray != '-1') {
                            throw new Exception("上传的电池条码在原托盘[{$original_tray}]是激活状态");
                        }
                    }
                }

                // 3、组装BKV的新增数据
                $insert_bkv_vals_chenge = [
                    "'{$battery_id}'",
                    "'{$tray}'",
                    "'{$lot}'",
                    $battery_id === '0' ? 0 : 1,
                    "'{$current_process_code}'",
                    $current_process_idx,
                    "'{$next_process_code}'",
                    "'{$pid}'",
                    "'{$uuid}'"
                ];
                if (!empty($insert_bkv_keys_change)) {
                    foreach ($result_param_need as $item) {
                        // 结果参数数据类型
                        $p_class = $result_param_need_column[$item['code']]['field_type'];
                        if (in_array($p_class, ['INT', 'FLOAT'])) {
                            array_push($insert_bkv_vals_chenge, $param[$item['code']][$battery_index]);
                        } else {
                            array_push($insert_bkv_vals_chenge, "'{$param[$item['code']][$battery_index]}'");
                        }
                    }
                }
                array_push($insert_bkv_vals, "(" . implode(',', $insert_bkv_vals_chenge) . ")");

                // 过滤掉0的情况
                if ($battery_id !== '0') {
                    // 4、组装battery_map的新增数据
                    $insert_battery_map_vals_chenge = [
                        "'{$batch}_{$subbatch}_{$tray}_{$lot}_{$battery_id}'",
                        "'{$batch}'",
                        "'{$subbatch}'",
                        "'{$tray}'",
                        "'{$lot}'",
                        "'{$battery_id}'",
                        "'{$pid}'",
                        "'{$uuid}'"
                    ];
                    array_push($insert_battery_map_vals, "(" . implode(',', $insert_battery_map_vals_chenge) . ")");

                    $forward_tracing_uuid = [];
                    //读取这个电池在process_log表中最新一条记录的forward_tracing_uuid，如果有的话没有的话则赋值uuid
                    $sql = "SELECT forward_tracing_uuid 
                        FROM hf_mes_bkv_batch_process_log 
                        WHERE item_id = '" . $battery_id . "' 
                        ORDER BY id DESC 
                        LIMIT 1";
                    $ret = Db::fetch($sql);
                    if (!empty($ret)) {
                        $forward_tracing_uuid = $ret['forward_tracing_uuid'];
                    }
                    // 5、组装process_log的新增数据
                    array_push($insert_process_log_data, [
                        'item_id' => $battery_id,
                        'batch' => $batch,
                        'subbatch' => $subbatch,
                        'tray' => $tray,
                        'lot' => $lot,
                        'process_code' => $current_process_code,
                        'workingsubclass' => $workingsubclass,
                        'device_code' => $device_code,
                        'channel' => $param['CHANNEL'][$battery_index],
                        'start_time' => $param['START_TIME'][$battery_index],
                        'finish_time' => $param['FINISH_TIME'][$battery_index],
                        'status' => -1,
                        'pid' => $pid,
                        'uuid' => $uuid,
                        'last_status' => 1,
                        'forward_tracing_uuid' => $forward_tracing_uuid ?: $uuid
                    ]);

                    $actual_input_battery_count++;
                }
            }

            // 组装tray_map表date_log数组
            $date_log = [];
            foreach ($flow_process as $item) {
                array_push($date_log, [
                    'idx' => $item[0],
                    'flow_process_id' => $item[1],
                    'flow_process_code' => $item[2],
                    'beginTime' => $current_process_code == $item[2] ? $param['START_TIME'][0] : '',
                    'endTime' => $current_process_code == $item[2] ? $param['FINISH_TIME'][0] : '',
                    'device_code' => $current_process_code == $item[2] ? $device_code : ''
                ]);
            }
        } catch (Exception $e) {
            throw new Exception($e->getMessage());
        }

        Db::beginTrans();

        try {
            // 标记最后一条数据的状态
            Event::emit('SetProcessLogEvent.updateLastStatus', $battery_ids);

            $input_battery_count = count($battery_ids);

            // 行锁tray_map表（注意：超时时此处行锁失效，tray_map表需要加上唯一索引unique_tray_map做约束）
            $sql = sprintf(
                "SELECT id
                    FROM hf_mes_production_tray_map
                    WHERE tray='%s' AND active='%s' LIMIT 1 FOR UPDATE;",
                $tray,
                1
            );
            $ret = Db::fetch($sql);
            if (!empty($ret)) {
                throw new Exception("托盘[{$tray}]已是激活状态");
            }


            // 写入tray_map数据
            $row = Db::insert('hf_mes_production_tray_map', [
                'tray_unique_value' => "{$batch}_{$subbatch}_{$tray}_{$lot}", // 数据库要做唯一索引，目的防止登录超时写入多条数据
                'batch' => $batch,
                'subbatch' => $subbatch,
                'tray' => $tray,
                'lot' => $lot,
                'active' => 1,
                'process_code' => $current_process_code,
                'process_idx' => $current_process_idx,
                'next_process_code' => $next_process_code,
                'date_log' => json_encode($date_log, JSON_UNESCAPED_UNICODE),
                'create_time' => $param['FINISH_TIME'][0],
                'input_battery_count' => $input_battery_count,
                'actual_input_battery_count' => $actual_input_battery_count
            ]);
            if ($row === NULL) {
                throw new Exception("写入tray_map数据失败");
            }

            // 写入BKV数据
            $sql = sprintf(
                "INSERT INTO \"%s\" %s VALUES %s;",
                config('app.bkv_prefix') . $subbatch,
                "({$insert_bkv_keys})",
                implode(',', $insert_bkv_vals)
            );
            $row = Db::query($sql);
            if ($row === NULL) {
                throw new Exception("写入BKV数据失败");
            }

            // 写入battery_map数据
            $sql = sprintf(
                "INSERT INTO \"%s\" %s VALUES %s;",
                'hf_mes_production_battery_map',
                "({$insert_battery_map_keys})",
                implode(',', $insert_battery_map_vals)
            );
            $row = Db::query($sql);
            if ($row === NULL) {
                throw new Exception("写入battery_map数据失败");
            }

            //批次表增加投入电池数
            $sql = sprintf(
                "UPDATE 
                    \"hf_mes_production_planning_management_batch\" SET input_amount = input_amount + %s 
                    WHERE batch='%s' ;",
                $input_battery_count,
                $batch
            );

            $ret = Db::query($sql);
            if ($ret < 1) {
                throw new Exception("批次[{$batch}]投入电芯数更新失败");
            }

            // 写入process_log数据(同步)
            Event::emit('SetProcessLogEvent.handle', $insert_process_log_data);

            // 判断是否超出了分表的数量限制，如果超出，则新增一个子批次
            Event::emit('CreateBkvEvent.handle', [
                'batch' => $batch,
                'subbatch' => $subbatch,
                'flow_id' => $flow_id,
                'batch_id' => $batch_id
            ]);
        } catch (Exception $e) {
            Db::rollBackTrans();
            throw new Exception($e->getMessage());
        }

        Db::commitTrans();

        return '';
    }
}
