<?php

namespace libs\core;

use libs\core\Config;
use libs\db\Db;
use Workerman\Worker;
use DIRECTORY_SEPARATOR;

class App
{
    /**
     * start
     */
    public static function run()
    {
        header("Access-Control-Allow-Origin: *"); // 允许任意域名发起的跨域请求
        header('Content-Type: text/html; charset=utf-8');
        header('Access-Control-Allow-Headers: Origin, X-Requested-With, Content-Type, Accept');
        header('Access-Control-Allow-Methods: GET, POST, PUT,OPTIONS');
        ini_set('display_errors', 'on');
        ini_set('display_errors', 'on');
        error_reporting(E_ALL);

        // 加载配置文件
        static::loadAllConfig();

        // 设置时区
        if ($timezone = config('app.default_timezone')) {
            date_default_timezone_set($timezone);
        }

        // 启动worker
        $config = config('server');

        if ($config['listen']) {
            $worker      = new Worker($config['listen'], $config['context']);
            $propertyMap = [
                'name',
                'count'
            ];
            foreach ($propertyMap as $property) {
                if (isset($config[$property])) {
                    $worker->$property = $config[$property];
                }
            }

            $worker->onWorkerStart = function ($worker) {
                // 加载数据库实例
                Db::instance();
                // 自动加载配置的文件
                foreach (config('autoload.files', []) as $file) {
                    include_once $file;
                }
                $init                   = new \libs\core\InterfaceForEquitment();
                $worker->onMessage      = [$init, 'onMessage'];
                $worker->onWorkerReload = [$init, 'onWorkerReload'];
                call_user_func([$init, 'onWorkerStart'], $worker);
            };
        }

        // 自定义进程，win下无效
        if (DIRECTORY_SEPARATOR === '/') {
            foreach (config('process', []) as $process_name => $config) {
                worker_start($process_name, $config);
            }
        }

        Worker::runAll();
    }

    /**
     * LoadAllConfig.
     * @param array $excludes
     * @return void
     */
    public static function loadAllConfig(array $excludes = [])
    {
        Config::load(config_path(), $excludes);
    }
}
