<?php

namespace libs\core;

use Exception;
use libs\db\Db;
use libs\core\Async;
use Workerman\Worker;
use Workerman\Connection\TcpConnection;
use Workerman\Protocols\Http\Request;
use Workerman\Protocols\Http\Response;

class InterfaceForEquitment
{
    protected static $worker = null;

    public function __construct()
    {
    }

    public function onWorkerStart(Worker $worker)
    {
        static::$worker = $worker;
        // 系统授权次数（达到授权次数，但还未达到授权时间，会更新一次上次检查时间last_date_check的值，防止篡改服务器时间来绕过系统授权验证）
        global $g_interface_auth_time;
        $g_interface_auth_time  = 10000;
    }

    public function onMessage(TcpConnection $connection, Request $request)
    {
        $response = new Response(200, [
            'Access-Control-Allow-Credentials' => 'true',
            'Access-Control-Allow-Origin'      => $request->header('Origin', '*'),
            'Access-Control-Allow-Methods'     => '*',
            'Access-Control-Allow-Headers'     => '*',
            'Content-Type'                     => 'application/json;charset=utf-8',
        ]);
        // 初始化接口请求变量
        $action = null;
        $post   = null;
        // 接口开始时刻
        $interface_start_time = microtime(true);
        // 客户断IP
        $client_ip = $connection->getRemoteIp();
        
        // 返回值只有''与对象方式
        list($rtval, $ext_script) = ['', ''];
        try {
            $body = $request->rawBody();

            if ($body === '') {
                throw new \Exception('空请求', 401);
            } else {
                $post = json_decode($body, true);
                if (json_last_error() !== JSON_ERROR_NONE) {
                    $post = $body;
                    throw new \Exception('POST的内容不是JSON，也并非可按照JSON解析的字符串！或者检查一下你的JSON是不是有注释？', 401);
                }
            }

            if (empty($post)) {
                throw new \Exception('空请求', 401);
            }
            if (!isset($post['action'])) {
                throw new \Exception('空action请求', 401);
            }

            // 判断param参数是否上传
            if (!isset($post['param'])) {
                throw new \Exception('缺少param参数', 401);
            }

            // 判断是否存在action类
            $action             = $post['action'];
            $action_explode_arr = explode('_', $action);
            $action_class       = '';
            foreach ($action_explode_arr as $item) {
                if (!empty($item)) {
                    $action_class .= ucfirst($item); 
                }
            }
            $action_class = "\\app\\action\\" . $action_class;
            if (!class_exists($action_class)) {
                throw new \Exception('未知action请求', 401);
            }


            // 接口系统授权调用验证,调用 checkAndUpdateLicense 函数，获取返回结果
            $licenseCheckResult = $this->checkAndUpdateLicense();
            // 检查返回值的 errcode，判断是否成功
            if ($licenseCheckResult['errcode'] != 0) {
                // 授权失败，返回失败信息
                throw new \Exception($licenseCheckResult['errmsg'], $licenseCheckResult['errcode']);
            }

            // 实例化对象
            $action_class_new = new $action_class;

            // 判断回调是否有效
            if (!is_callable([$action_class_new, 'execute'])) {
                throw new Exception("接口异常，请检查接口是否包含execute方法");
            }
            $body = call_user_func([$action_class_new, 'execute'], $post);

            if (empty($body)) {
                $body = '';
            } elseif (is_array($body)) {
                if (isset($body['rtval'])) {
                    if (is_array($body['rtval'])) {
                        $rtval = (object)$body['rtval'];
                    } else {
                        $rtval = $body['rtval'];
                    }
                } else {
                    $rtval = (object)$body;
                }
                $ext_script = isset($body['ext_script']) && !empty($body['ext_script']) ? (object)$body['ext_script'] : '';
            }elseif (is_object($body)) {
                $rtval = $body;
            }
            $result = array(
                'action'  => 'result_' . $action,
                'errcode' => 0,
                'errmsg'  => '',
                'rtval'   => $rtval
            );
            // 如果计算脚本的开关关闭，则不返回计算脚本的内容
            if (config('mes.calculation_script_switch')) {
                $result['ext_script'] = $ext_script;
            }

            $response->withBody(json_encode($result));
        } catch (Exception $e) {
            $errcode = 4001;

            if (is_int($e->getCode()) && $e->getCode() != 0) {
                $errcode = $e->getCode();
            }
            $result = array(
                'action'  => 'result_' . $action,
                'errcode' => $errcode,
                'errmsg'  => $e->getMessage(),
                'rtval'   => ''
            );
            // 如果计算脚本的开关关闭，则不返回计算脚本的内容
            if (config('mes.calculation_script_switch')) {
                $result['ext_script'] = $ext_script;
            }

            $response->withBody(json_encode($result));
        }

        // 记录接口日志
        $this->logRecord($interface_start_time, $client_ip, $post, $result);

        $connection->send($response);
    }

    public function onWorkerReload(Worker $worker)
    {
        // 加载数据库实例
        Db::instance();
    }

    private function logRecord($interface_start_time, $client_ip, $post, $result)
    {
        // 接口结束时刻
        $interface_end_time = microtime(true);
        $insterface_time    = floatval(round($interface_end_time - $interface_start_time, 3) * 1000);

        // 判断来源
        $class = 'workerman';
        if (isset($post['action'])) {
            $erp_list = [
                'set_material',
                'set_purchase_order',
                'set_supplier',
                'set_sublibrary',
                'set_invoice',
                'set_material_outbound_result',
                'set_purchase_receive_result',
                'set_purchase_examine_result',
                'set_purchase_inbound_result',
                'set_material_bom',
            ];
            if (in_array($post['action'], $erp_list)) {
                $class = 'erp';
            }
        } else {
            $class = 'erp';
        }

        // 写入日志
        $data                    = array();
        $data['class']           = $class;
        $data['unit']            = isset($post['action']) && !empty($post['action']) ? $post['action'] : '';
        $data['status']          = $result['errcode'] > 0 ? $result['errcode'] : 0;
        $data['insterface_time'] = $insterface_time;
        $data['params']          = json_encode($post, JSON_UNESCAPED_UNICODE);
        $data['result']          = json_encode($result, JSON_UNESCAPED_UNICODE);
        $data['client_ip']       = $client_ip;
        $data['data1']           = isset($post['process_code']) && !empty($post['process_code']) ? $post['process_code'] : '';
        $data['data2']           = isset($post['param']['tray']) && !empty($post['param']['tray']) ? $post['param']['tray'] : '';
        $data3                   = '';
        switch ($data['unit']) {
            case "set_battery_inactivity":
                if (isset($post['param']['data'][0]['battery_id']) && !empty($post['param']['data'][0]['battery_id'])) {
                    $data3 = $post['param']['data'][0]['battery_id'];
                } elseif (isset($post['param']['battery_ids']) && !empty($post['param']['battery_ids'])) {
                    if (is_array($post['param']['battery_ids'])) {
                        $data3 = implode(',', $post['param']['battery_ids']);
                    } else {
                        $data3 = $post['param']['battery_ids'];
                    }
                }
                break;
            case "set_tray_inactivity":
                if (isset($post['param']['data']) && !empty($post['param']['data'])) {
                    $data['data2'] = implode(',', array_column($post['param']['data'], 'tray'));
                }
                break;
            default:
                if (isset($post['param']['battery_ids']) && !empty($post['param']['battery_ids']) && is_array($post['param']['battery_ids'])) {
                    $data3 = implode(",", $post['param']['battery_ids']);
                }
        }
        $data['data3'] = $data3;

        // 异步写入日志
        Async::asyncTaskConnection(config('app.log_server'), $data);
    }

    function getServerIp($request)
    {
        $ip = null;
        if (isset($_SERVER['SERVER_ADDR'])) {
            $ip = $_SERVER['SERVER_ADDR'];
        } else {
            if (isset($_SERVER['SERVER_NAME'])) {
                $ip = $_SERVER['SERVER_NAME'];
            }
        }
        return $ip;
    }

    // 系统授权验证
    function checkAndUpdateLicense($licenseFile = 'license.license') {
        // 返回的结果数组，默认成功
        $result = [
            'errcode' => 0,  // 0 表示成功
            'errmsg' => '操作成功'
        ];
        // 第 1 步: 解压 license.license，获取 license.dat 文件内容
        $licenseData = '';
        $zip = new \ZipArchive();
    
        if ($zip->open($licenseFile) === TRUE) {
            // 解压到临时目录
            $tempDir = sys_get_temp_dir() . '/license_data/';
            if (!is_dir($tempDir)) {
                mkdir($tempDir, 0777, true);
            }
            $zip->extractTo($tempDir);  // 解压文件
            $zip->close();
    
            // 获取解压后的 license.dat 文件内容
            $licenseData = file_get_contents($tempDir . 'license.dat');
        } else {
            $result['errcode'] = 1;
            $result['errmsg'] = "无法打开授权文件 license.license";
            return $result;
        }
    
        // 第 2 步: 将 license.dat 内容转化为 JSON
        $licenseJson = json_decode($licenseData, true);
        if (!$licenseJson) {
            $result['errcode'] = 2;
            $result['errmsg'] = "无法解析 license.dat 内容为 JSON";
            return $result;
        }
        // 第 3 步: 获取 expiration_date 和 last_date_check
        $expirationDate = new \DateTime($licenseJson['expiration_date']);
        $lastDateCheck = new \DateTime($licenseJson['last_date_check']);
        $currentDate = new \DateTime();
    
        // 第 4 步: 检查当前时间是否早于 last_date_check
        if ($currentDate <= $lastDateCheck) {
            $result['errcode'] = 3;
            $result['errmsg'] = "当前时间早于上次检查时间，请检查授权文件。";
            // 删除 license.license 文件
            unlink($licenseFile);
            return $result;
        }else{
            // 第 5 步: 如果当前时间比 last_date_check 晚，继续判断
            // 第 6 步: 判断当前时间是否已经超过 expiration_date
            if ($currentDate > $expirationDate) {
                $result['errcode'] = 4;
                $result['errmsg'] = "授权已过期，系统调用失败。";
                // 删除 license.license 文件
                unlink($licenseFile);
                return $result;
            }
            
            global $g_interface_auth_time;
            if($g_interface_auth_time>=10000){

                // 第 7 步: 如果授权未过期，更新 last_date_check 为当前时间
                $licenseJson['last_date_check'] = $currentDate->format('Y-m-d H:i:s');
                
                // 第 8 步: 将更新后的 JSON 数据加密回 license.license 中的 license.dat
                // 将更新后的 JSON 编码
                $updatedLicenseData = json_encode($licenseJson, JSON_PRETTY_PRINT);
                
                // 创建临时文件用于存储更新后的 license.dat 内容
                $tempFile = tempnam(sys_get_temp_dir(), 'license_');
                file_put_contents($tempFile, $updatedLicenseData);  // 写入更新后的 JSON 数据
        
                // 创建新的 ZIP 文件，添加加密后的 license.dat 文件
                $zip->open($licenseFile, \ZipArchive::CREATE);
                $zip->addFile($tempFile, 'license.dat');
                $zip->close();
        
                // 删除临时文件
                unlink($tempFile);
        
                $result['errmsg'] = "授权文件已更新并加密成功。";
                
                $g_interface_auth_time = 0;
            }else{
                ++$g_interface_auth_time ;
            }

        }

        return $result;
    }

}
